@@ -5,9 +5,11 @@ Expose all functions as pointers in a dedicated C extension.
 #define NUMBA_EXPORT_FUNC(_rettype) static _rettype
 #define NUMBA_EXPORT_DATA(_vartype) static _vartype
 
+/* Import _pymodule.h first, for a recent _POSIX_C_SOURCE */
+#include "_pymodule.h"
+#include <math.h>
 #include "_helperlib.c"
 
-
 static PyObject *
 build_c_helpers_dict(void)
 {
@@ -101,6 +103,30 @@ static PyMethodDef ext_methods[] = {
     { NULL },
 };
 
+/*
+ * These functions are exported by the module's DLL, to exercise ctypes / cffi
+ * without relying on libc availability (see https://bugs.python.org/issue23606)
+ */
+
+PyAPI_FUNC(double) _numba_test_sin(double x);
+PyAPI_FUNC(double) _numba_test_cos(double x);
+PyAPI_FUNC(double) _numba_test_exp(double x);
+
+double _numba_test_sin(double x)
+{
+    return sin(x);
+}
+
+double _numba_test_cos(double x)
+{
+    return cos(x);
+}
+
+double _numba_test_exp(double x)
+{
+    return exp(x);
+}
+
 
 MOD_INIT(_helperlib) {
     PyObject *m;
