@@ -378,7 +378,7 @@ class UniTupleModel(DataModel):
     def traverse(self, builder, value):
         values = cgutils.unpack_tuple(builder, value, count=self._count)
         return zip([self._fe_type.dtype] * len(values), values)
-    
+
     def inner_types(self):
         return self._elem_model.traverse_types()
 
@@ -399,6 +399,13 @@ class StructModel(CompositeModel):
             self._fields = self._members = ()
         self._models = tuple([self._dmm.lookup(t) for t in self._members])
 
+    def get_member_fe_type(self, name):
+        """
+        StructModel-specific: get the Numba type of the field named *name*.
+        """
+        pos = self.get_field_position(name)
+        return self._members[pos]
+
     def get_value_type(self):
         elems = [t.get_value_type() for t in self._models]
         return ir.LiteralStructType(elems)
@@ -572,6 +579,9 @@ class StructModel(CompositeModel):
         return self._models[pos]
 
     def traverse(self, builder, value):
+        if value.type != self.get_value_type():
+            args = self.get_value_type(), value.type
+            raise TypeError("expecting {0} but got {1}".format(*args))
         out = [(self.get_type(k), self.get(builder, value, k))
                 for k in self._fields]
         return out
@@ -840,13 +850,14 @@ class UniTupleIter(StructModel):
         super(UniTupleIter, self).__init__(dmm, fe_type, members)
 
 
-@register_default(types.Slice3Type)
-class Slice3(StructModel):
+@register_default(types.SliceType)
+class SliceModel(StructModel):
     def __init__(self, dmm, fe_type):
         members = [('start', types.intp),
                    ('stop', types.intp),
-                   ('step', types.intp)]
-        super(Slice3, self).__init__(dmm, fe_type, members)
+                   ('step', types.intp),
+                   ]
+        super(SliceModel, self).__init__(dmm, fe_type, members)
 
 
 @register_default(types.NPDatetime)
@@ -995,3 +1006,83 @@ def handle_numpy_ndenumerate_type(dmm, ty):
 def handle_bound_function(dmm, ty):
     # The same as the underlying type
     return dmm[ty.this]
+
+
+@register_default(types.DeferredType)
+class DeferredStructModel(CompositeModel):
+    def __init__(self, dmm, fe_type):
+        super(DeferredStructModel, self).__init__(dmm, fe_type)
+        self.typename = "deferred.{0}".format(id(fe_type))
+        self.actual_fe_type = fe_type.get()
+
+    def get_value_type(self):
+        return ir.global_context.get_identified_type(self.typename + '.value')
+
+    def get_data_type(self):
+        return ir.global_context.get_identified_type(self.typename + '.data')
+
+    def get_argument_type(self):
+        return self._actual_model.get_argument_type()
+
+    def as_argument(self, builder, value):
+        inner = self.get(builder, value)
+        return self._actual_model.as_argument(builder, inner)
+
+    def from_argument(self, builder, value):
+        res = self._actual_model.from_argument(builder, value)
+        return self.set(builder, self.make_uninitialized(), res)
+
+    def from_data(self, builder, value):
+        self._define()
+        elem = self.get(builder, value)
+        value = self._actual_model.from_data(builder, elem)
+        out = self.make_uninitialized()
+        return self.set(builder, out, value)
+
+    def as_data(self, builder, value):
+        self._define()
+        elem = self.get(builder, value)
+        value = self._actual_model.as_data(builder, elem)
+        out = self.make_uninitialized(kind='data')
+        return self.set(builder, out, value)
+
+    def from_return(self, builder, value):
+        return value
+
+    def as_return(self, builder, value):
+        return value
+
+    def get(self, builder, value):
+        return builder.extract_value(value, [0])
+
+    def set(self, builder, value, content):
+        return builder.insert_value(value, content, [0])
+
+    def make_uninitialized(self, kind='value'):
+        self._define()
+        if kind == 'value':
+            ty = self.get_value_type()
+        else:
+            ty = self.get_data_type()
+        return ir.Constant(ty, ir.Undefined)
+
+    def _define(self):
+        valty = self.get_value_type()
+        self._define_value_type(valty)
+        datty = self.get_data_type()
+        self._define_data_type(datty)
+
+    def _define_value_type(self, value_type):
+        if value_type.is_opaque:
+            value_type.set_body(self._actual_model.get_value_type())
+
+    def _define_data_type(self, data_type):
+        if data_type.is_opaque:
+            data_type.set_body(self._actual_model.get_data_type())
+
+    @property
+    def _actual_model(self):
+        return self._dmm.lookup(self.actual_fe_type)
+
+    def traverse(self, builder, value):
+        return [(self.actual_fe_type, builder.extract_value(value, [0]))]
