@@ -20,11 +20,11 @@ CC_SPIR_FUNC = "spir_func"
 
 
 class HSATypingContext(typing.BaseContext):
-    def init(self):
+    def load_additional_registries(self):
         from . import hsadecl, mathdecl
 
-        self.install(hsadecl.registry)
-        self.install(mathdecl.registry)
+        self.install_registry(hsadecl.registry)
+        self.install_registry(mathdecl.registry)
 
 
 # -----------------------------------------------------------------------------
@@ -64,15 +64,17 @@ class HSATargetContext(BaseContext):
     generic_addrspace = SPIR_GENERIC_ADDRSPACE
 
     def init(self):
-        from . import hsaimpl, mathimpl
-
-        self.insert_func_defn(hsaimpl.registry.functions)
-        self.insert_func_defn(mathimpl.registry.functions)
         self._internal_codegen = codegen.JITHSACodegen("numba.hsa.jit")
         self._target_data = DATALAYOUT[utils.MACHINE_BITS]
         # Override data model manager
         self.data_model_manager = hsa_data_model_manager
 
+    def load_additional_registries(self):
+        from . import hsaimpl, mathimpl
+
+        self.insert_func_defn(hsaimpl.registry.functions)
+        self.insert_func_defn(mathimpl.registry.functions)
+
     @cached_property
     def call_conv(self):
         return HSACallConv(self)
@@ -115,8 +117,11 @@ class HSATargetContext(BaseContext):
                         lty.addrspace)
             return lty, None
 
-        llargtys, changed = zip(*map(sub_gen_with_global,
-                                     arginfo.argument_types))
+        if len(arginfo.argument_types) > 0:
+            llargtys, changed = zip(*map(sub_gen_with_global,
+                                         arginfo.argument_types))
+        else:
+            llargtys = changed = ()
         wrapperfnty = lc.Type.function(lc.Type.void(), llargtys)
 
         wrapper_module = self.create_module("hsa.kernel.wrapper")
