@@ -7,15 +7,14 @@ from llvmlite import ir
 from llvmlite.llvmpy.core import Type, Constant
 import llvmlite.llvmpy.core as lc
 
-from .imputils import (builtin, builtin_attr, implement, impl_attribute,
-                       impl_attribute_generic, iternext_impl,
+from .imputils import (lower_builtin, lower_getattr, lower_getattr_generic,
+                       lower_cast, iternext_impl,
                        impl_ret_borrowed, impl_ret_untracked)
 from . import optional
 from .. import typing, types, cgutils, utils, intrinsics
 
 
-@builtin
-@implement('is not', types.Any, types.Any)
+@lower_builtin('is not', types.Any, types.Any)
 def generic_is_not(context, builder, sig, args):
     """
     Implement `x is not y` as `not (x is y)`.
@@ -115,9 +114,8 @@ def int_divmod(context, builder, x, y):
     return resdiv, resmod
 
 
-@builtin
-@implement('/?', types.Kind(types.Integer), types.Kind(types.Integer))
-@implement('//', types.Kind(types.Integer), types.Kind(types.Integer))
+@lower_builtin('/?', types.Integer, types.Integer)
+@lower_builtin('//', types.Integer, types.Integer)
 def int_floordiv_impl(context, builder, sig, args):
     [va, vb] = args
     [ta, tb] = sig.args
@@ -145,8 +143,7 @@ def int_floordiv_impl(context, builder, sig, args):
                               builder.load(res))
 
 
-@builtin
-@implement('/', types.Kind(types.Integer), types.Kind(types.Integer))
+@lower_builtin('/', types.Integer, types.Integer)
 def int_truediv_impl(context, builder, sig, args):
     [va, vb] = args
     [ta, tb] = sig.args
@@ -158,8 +155,7 @@ def int_truediv_impl(context, builder, sig, args):
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
 
-@builtin
-@implement('%', types.Kind(types.Integer), types.Kind(types.Integer))
+@lower_builtin('%', types.Integer, types.Integer)
 def int_rem_impl(context, builder, sig, args):
     [va, vb] = args
     [ta, tb] = sig.args
@@ -378,6 +374,9 @@ def bool_invert_impl(context, builder, sig, args):
 
 
 def int_sign_impl(context, builder, sig, args):
+    """
+    np.sign(int)
+    """
     [x] = args
     POS = Constant.int(x.type, 1)
     NEG = Constant.int(x.type, -1)
@@ -416,57 +415,55 @@ def int_sign_impl(context, builder, sig, args):
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
 
-builtin(implement('==', types.boolean, types.boolean)(int_eq_impl))
-builtin(implement('!=', types.boolean, types.boolean)(int_ne_impl))
-builtin(implement('<', types.boolean, types.boolean)(int_ult_impl))
-builtin(implement('<=', types.boolean, types.boolean)(int_ule_impl))
-builtin(implement('>', types.boolean, types.boolean)(int_ugt_impl))
-builtin(implement('>=', types.boolean, types.boolean)(int_uge_impl))
-builtin(implement('~', types.boolean)(bool_invert_impl))
+lower_builtin('==', types.boolean, types.boolean)(int_eq_impl)
+lower_builtin('!=', types.boolean, types.boolean)(int_ne_impl)
+lower_builtin('<', types.boolean, types.boolean)(int_ult_impl)
+lower_builtin('<=', types.boolean, types.boolean)(int_ule_impl)
+lower_builtin('>', types.boolean, types.boolean)(int_ugt_impl)
+lower_builtin('>=', types.boolean, types.boolean)(int_uge_impl)
+lower_builtin('~', types.boolean)(bool_invert_impl)
 
 
 def _implement_integer_operators():
-    ty = types.Kind(types.Integer)
+    ty = types.Integer
 
-    builtin(implement('+', ty, ty)(int_add_impl))
-    builtin(implement('-', ty, ty)(int_sub_impl))
-    builtin(implement('*', ty, ty)(int_mul_impl))
-    builtin(implement('==', ty, ty)(int_eq_impl))
-    builtin(implement('!=', ty, ty)(int_ne_impl))
+    lower_builtin('+', ty, ty)(int_add_impl)
+    lower_builtin('-', ty, ty)(int_sub_impl)
+    lower_builtin('*', ty, ty)(int_mul_impl)
+    lower_builtin('==', ty, ty)(int_eq_impl)
+    lower_builtin('!=', ty, ty)(int_ne_impl)
 
-    builtin(implement('<<', ty, ty)(int_shl_impl))
-    builtin(implement('>>', ty, ty)(int_shr_impl))
+    lower_builtin('<<', ty, ty)(int_shl_impl)
+    lower_builtin('>>', ty, ty)(int_shr_impl)
 
-    builtin(implement('&', ty, ty)(int_and_impl))
-    builtin(implement('|', ty, ty)(int_or_impl))
-    builtin(implement('^', ty, ty)(int_xor_impl))
+    lower_builtin('&', ty, ty)(int_and_impl)
+    lower_builtin('|', ty, ty)(int_or_impl)
+    lower_builtin('^', ty, ty)(int_xor_impl)
 
-    builtin(implement('-', ty)(int_negate_impl))
-    builtin(implement('+', ty)(int_positive_impl))
-    builtin(implement(types.neg_type, ty)(int_negate_impl))
-    builtin(implement('~', ty)(int_invert_impl))
-    builtin(implement(types.sign_type, ty)(int_sign_impl))
+    lower_builtin('-', ty)(int_negate_impl)
+    lower_builtin('+', ty)(int_positive_impl)
+    lower_builtin('~', ty)(int_invert_impl)
 
-    builtin(implement('**', ty, ty)(int_power_impl))
-    builtin(implement(pow, ty, ty)(int_power_impl))
+    lower_builtin('**', ty, ty)(int_power_impl)
+    lower_builtin(pow, ty, ty)(int_power_impl)
 
     for ty in types.unsigned_domain:
-        builtin(implement('<', ty, ty)(int_ult_impl))
-        builtin(implement('<=', ty, ty)(int_ule_impl))
-        builtin(implement('>', ty, ty)(int_ugt_impl))
-        builtin(implement('>=', ty, ty)(int_uge_impl))
-        builtin(implement('**', types.float64, ty)(int_power_impl))
-        builtin(implement(pow, types.float64, ty)(int_power_impl))
-        builtin(implement(types.abs_type, ty)(uint_abs_impl))
+        lower_builtin('<', ty, ty)(int_ult_impl)
+        lower_builtin('<=', ty, ty)(int_ule_impl)
+        lower_builtin('>', ty, ty)(int_ugt_impl)
+        lower_builtin('>=', ty, ty)(int_uge_impl)
+        lower_builtin('**', types.float64, ty)(int_power_impl)
+        lower_builtin(pow, types.float64, ty)(int_power_impl)
+        lower_builtin(abs, ty)(uint_abs_impl)
 
     for ty in types.signed_domain:
-        builtin(implement('<', ty, ty)(int_slt_impl))
-        builtin(implement('<=', ty, ty)(int_sle_impl))
-        builtin(implement('>', ty, ty)(int_sgt_impl))
-        builtin(implement('>=', ty, ty)(int_sge_impl))
-        builtin(implement(types.abs_type, ty)(int_abs_impl))
-        builtin(implement('**', types.float64, ty)(int_power_impl))
-        builtin(implement(pow, types.float64, ty)(int_power_impl))
+        lower_builtin('<', ty, ty)(int_slt_impl)
+        lower_builtin('<=', ty, ty)(int_sle_impl)
+        lower_builtin('>', ty, ty)(int_sgt_impl)
+        lower_builtin('>=', ty, ty)(int_sge_impl)
+        lower_builtin('**', types.float64, ty)(int_power_impl)
+        lower_builtin(pow, types.float64, ty)(int_power_impl)
+        lower_builtin(abs, ty)(int_abs_impl)
 
 _implement_integer_operators()
 
@@ -500,15 +497,45 @@ def optional_is_not_none(context, builder, sig, args):
 
 
 # None is/not None
-builtin(implement('is', types.none, types.none)(
-    optional.always_return_true_impl))
+lower_builtin('is', types.none, types.none)(optional.always_return_true_impl)
 
 # Optional is None
-builtin(implement('is',
-                  types.Kind(types.Optional), types.none)(optional_is_none))
+lower_builtin('is', types.Optional, types.none)(optional_is_none)
+lower_builtin('is', types.none, types.Optional)(optional_is_none)
 
-builtin(implement('is',
-                  types.none, types.Kind(types.Optional))(optional_is_none))
+
+@lower_getattr_generic(types.Optional)
+def optional_getattr(context, builder, typ, value, attr):
+    """
+    Optional.__getattr__ => redirect to the wrapped type.
+    """
+    inner_type = typ.type
+    val = context.cast(builder, value, typ, inner_type)
+    imp = context.get_getattr(inner_type, attr)
+    return imp(context, builder, inner_type, val, attr)
+
+
+@lower_getattr_generic(types.DeferredType)
+def deferred_getattr(context, builder, typ, value, attr):
+    """
+    Deferred.__getattr__ => redirect to the actual type.
+    """
+    inner_type = typ.get()
+    val = context.cast(builder, value, typ, inner_type)
+    imp = context.get_getattr(inner_type, attr)
+    return imp(context, builder, inner_type, val, attr)
+
+@lower_cast(types.Any, types.DeferredType)
+def any_to_deferred(context, builder, fromty, toty, val):
+    actual = context.cast(builder, val, fromty, toty.get())
+    model = context.data_model_manager[toty]
+    return model.set(builder, model.make_uninitialized(), actual)
+
+@lower_cast(types.DeferredType, types.Any)
+def deferred_to_any(context, builder, fromty, toty, val):
+    model = context.data_model_manager[fromty]
+    val = model.get(builder, val)
+    return context.cast(builder, val, fromty.get(), toty)
 
 
 def real_add_impl(context, builder, sig, args):
@@ -765,6 +792,9 @@ def real_positive_impl(context, builder, sig, args):
 
 
 def real_sign_impl(context, builder, sig, args):
+    """
+    np.sign(float)
+    """
     [x] = args
     POS = Constant.real(x.type, 1)
     NEG = Constant.real(x.type, -1)
@@ -791,31 +821,29 @@ def real_sign_impl(context, builder, sig, args):
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
 
-ty = types.Kind(types.Float)
+ty = types.Float
 
-builtin(implement('+', ty, ty)(real_add_impl))
-builtin(implement('-', ty, ty)(real_sub_impl))
-builtin(implement('*', ty, ty)(real_mul_impl))
-builtin(implement('/?', ty, ty)(real_div_impl))
-builtin(implement('//', ty, ty)(real_floordiv_impl))
-builtin(implement('/', ty, ty)(real_div_impl))
-builtin(implement('%', ty, ty)(real_mod_impl))
-builtin(implement('**', ty, ty)(real_power_impl))
-builtin(implement(pow, ty, ty)(real_power_impl))
+lower_builtin('+', ty, ty)(real_add_impl)
+lower_builtin('-', ty, ty)(real_sub_impl)
+lower_builtin('*', ty, ty)(real_mul_impl)
+lower_builtin('/?', ty, ty)(real_div_impl)
+lower_builtin('//', ty, ty)(real_floordiv_impl)
+lower_builtin('/', ty, ty)(real_div_impl)
+lower_builtin('%', ty, ty)(real_mod_impl)
+lower_builtin('**', ty, ty)(real_power_impl)
+lower_builtin(pow, ty, ty)(real_power_impl)
 
-builtin(implement('==', ty, ty)(real_eq_impl))
-builtin(implement('!=', ty, ty)(real_ne_impl))
-builtin(implement('<', ty, ty)(real_lt_impl))
-builtin(implement('<=', ty, ty)(real_le_impl))
-builtin(implement('>', ty, ty)(real_gt_impl))
-builtin(implement('>=', ty, ty)(real_ge_impl))
+lower_builtin('==', ty, ty)(real_eq_impl)
+lower_builtin('!=', ty, ty)(real_ne_impl)
+lower_builtin('<', ty, ty)(real_lt_impl)
+lower_builtin('<=', ty, ty)(real_le_impl)
+lower_builtin('>', ty, ty)(real_gt_impl)
+lower_builtin('>=', ty, ty)(real_ge_impl)
 
-builtin(implement(types.abs_type, ty)(real_abs_impl))
+lower_builtin(abs, ty)(real_abs_impl)
 
-builtin(implement('-', ty)(real_negate_impl))
-builtin(implement('+', ty)(real_positive_impl))
-builtin(implement(types.neg_type, ty)(real_negate_impl))
-builtin(implement(types.sign_type, ty)(real_sign_impl))
+lower_builtin('-', ty)(real_negate_impl)
+lower_builtin('+', ty)(real_positive_impl)
 
 del ty
 
@@ -841,24 +869,21 @@ def get_complex_info(ty):
     return cmplxcls, ty.underlying_float
 
 
-@builtin_attr
-@impl_attribute(types.Kind(types.Complex), "real")
+@lower_getattr(types.Complex, "real")
 def complex_real_impl(context, builder, typ, value):
     cplx_cls = context.make_complex(typ)
     cplx = cplx_cls(context, builder, value=value)
     res = cplx.real
     return impl_ret_untracked(context, builder, typ, res)
 
-@builtin_attr
-@impl_attribute(types.Kind(types.Complex), "imag")
+@lower_getattr(types.Complex, "imag")
 def complex_imag_impl(context, builder, typ, value):
     cplx_cls = context.make_complex(typ)
     cplx = cplx_cls(context, builder, value=value)
     res = cplx.imag
     return impl_ret_untracked(context, builder, typ, res)
 
-@builtin
-@implement("complex.conjugate", types.Kind(types.Complex))
+@lower_builtin("complex.conjugate", types.Complex)
 def complex_conjugate_impl(context, builder, sig, args):
     from . import mathimpl
     cplx_cls = context.make_complex(sig.args[0])
@@ -878,14 +903,13 @@ def real_conjugate_impl(context, builder, sig, args):
     return impl_ret_untracked(context, builder, sig.return_type, args[0])
 
 for cls in (types.Float, types.Integer):
-    builtin_attr(impl_attribute(types.Kind(cls), "real")(real_real_impl))
-    builtin_attr(impl_attribute(types.Kind(cls), "imag")(real_imag_impl))
-    builtin(implement("complex.conjugate", types.Kind(cls))(real_conjugate_impl))
+    lower_getattr(cls, "real")(real_real_impl)
+    lower_getattr(cls, "imag")(real_imag_impl)
+    lower_builtin("complex.conjugate", cls)(real_conjugate_impl)
 
 
-@builtin
-@implement("**", types.complex128, types.complex128)
-@implement(pow, types.complex128, types.complex128)
+@lower_builtin("**", types.complex128, types.complex128)
+@lower_builtin(pow, types.complex128, types.complex128)
 def complex128_power_impl(context, builder, sig, args):
     [ca, cb] = args
     a = Complex128(context, builder, value=ca)
@@ -1063,21 +1087,21 @@ def complex_abs_impl(context, builder, sig, args):
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
 
-ty = types.Kind(types.Complex)
+ty = types.Complex
 
-builtin(implement("+", ty, ty)(complex_add_impl))
-builtin(implement("-", ty, ty)(complex_sub_impl))
-builtin(implement("*", ty, ty)(complex_mul_impl))
-builtin(implement("/?", ty, ty)(complex_div_impl))
-builtin(implement("/", ty, ty)(complex_div_impl))
-builtin(implement("-", ty)(complex_negate_impl))
-builtin(implement("+", ty)(complex_positive_impl))
+lower_builtin("+", ty, ty)(complex_add_impl)
+lower_builtin("-", ty, ty)(complex_sub_impl)
+lower_builtin("*", ty, ty)(complex_mul_impl)
+lower_builtin("/?", ty, ty)(complex_div_impl)
+lower_builtin("/", ty, ty)(complex_div_impl)
+lower_builtin("-", ty)(complex_negate_impl)
+lower_builtin("+", ty)(complex_positive_impl)
 # Complex modulo is deprecated in python3
 
-builtin(implement('==', ty, ty)(complex_eq_impl))
-builtin(implement('!=', ty, ty)(complex_ne_impl))
+lower_builtin('==', ty, ty)(complex_eq_impl)
+lower_builtin('!=', ty, ty)(complex_ne_impl)
 
-builtin(implement(types.abs_type, ty)(complex_abs_impl))
+lower_builtin(abs, ty)(complex_abs_impl)
 
 del ty
 
@@ -1093,26 +1117,22 @@ def number_not_impl(context, builder, sig, args):
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
 
-@builtin
-@implement(bool, types.boolean)
+@lower_builtin(bool, types.boolean)
 def bool_as_bool(context, builder, sig, args):
     [val] = args
     return val
 
-@builtin
-@implement(bool, types.Kind(types.Integer))
+@lower_builtin(bool, types.Integer)
 def int_as_bool(context, builder, sig, args):
     [val] = args
     return builder.icmp_unsigned('!=', val, ir.Constant(val.type, 0))
 
-@builtin
-@implement(bool, types.Kind(types.Float))
+@lower_builtin(bool, types.Float)
 def float_as_bool(context, builder, sig, args):
     [val] = args
     return builder.fcmp(lc.FCMP_UNE, val, ir.Constant(val.type, 0.0))
 
-@builtin
-@implement(bool, types.Kind(types.Complex))
+@lower_builtin(bool, types.Complex)
 def complex_as_bool(context, builder, sig, args):
     [typ] = sig.args
     [val] = args
@@ -1125,9 +1145,9 @@ def complex_as_bool(context, builder, sig, args):
 
 
 for ty in (types.Integer, types.Float, types.Complex):
-    builtin(implement('not', types.Kind(ty))(number_not_impl))
+    lower_builtin('not', ty)(number_not_impl)
 
-builtin(implement('not', types.boolean)(number_not_impl))
+lower_builtin('not', types.boolean)(number_not_impl)
 
 #------------------------------------------------------------------------------
 
@@ -1135,8 +1155,7 @@ def make_pair(first_type, second_type):
     return cgutils.create_struct_proxy(types.Pair(first_type, second_type))
 
 
-@builtin
-@implement('getitem', types.Kind(types.CPointer), types.Kind(types.Integer))
+@lower_builtin('getitem', types.CPointer, types.Integer)
 def getitem_cpointer(context, builder, sig, args):
     base_ptr, idx = args
     elem_ptr = builder.gep(base_ptr, [idx])
@@ -1144,8 +1163,7 @@ def getitem_cpointer(context, builder, sig, args):
     return impl_ret_borrowed(context, builder, sig.return_type, res)
 
 
-@builtin
-@implement('setitem', types.Kind(types.CPointer), types.Kind(types.Integer),
+@lower_builtin('setitem', types.CPointer, types.Integer,
            types.Any)
 def setitem_cpointer(context, builder, sig, args):
     base_ptr, idx, val = args
@@ -1155,21 +1173,7 @@ def setitem_cpointer(context, builder, sig, args):
 
 #-------------------------------------------------------------------------------
 
-@builtin
-@implement(types.NumberClass, types.Any)
-def number_constructor(context, builder, sig, args):
-    """
-    Convert any number to any other.
-    """
-    [val] = args
-    [valty] = sig.args
-    return context.cast(builder, val, valty, sig.return_type)
-
-
-#-------------------------------------------------------------------------------
-
-@builtin
-@implement(max, types.VarArg(types.Any))
+@lower_builtin(max, types.VarArg(types.Any))
 def max_impl(context, builder, sig, args):
     argtys = sig.args
     for a in argtys:
@@ -1193,8 +1197,7 @@ def max_impl(context, builder, sig, args):
     return impl_ret_borrowed(context, builder, sig.return_type, resval)
 
 
-@builtin
-@implement(min, types.VarArg(types.Any))
+@lower_builtin(min, types.VarArg(types.Any))
 def min_impl(context, builder, sig, args):
     argtys = sig.args
     for a in argtys:
@@ -1225,8 +1228,7 @@ def _round_intrinsic(tp):
     else:
         return "llvm.round.f%d" % (tp.bitwidth,)
 
-@builtin
-@implement(round, types.Kind(types.Float))
+@lower_builtin(round, types.Float)
 def round_impl_unary(context, builder, sig, args):
     fltty = sig.args[0]
     llty = context.get_value_type(fltty)
@@ -1239,8 +1241,7 @@ def round_impl_unary(context, builder, sig, args):
         res = builder.fptosi(res, context.get_value_type(sig.return_type))
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
-@builtin
-@implement(round, types.Kind(types.Float), types.Kind(types.Integer))
+@lower_builtin(round, types.Float, types.Integer)
 def round_impl_binary(context, builder, sig, args):
     fltty = sig.args[0]
     # Allow calling the intrinsic from the Python implementation below.
@@ -1276,9 +1277,9 @@ def round_impl_binary(context, builder, sig, args):
 
 
 #-------------------------------------------------------------------------------
+# Numeric constructors
 
-@builtin
-@implement(int, types.Any)
+@lower_builtin(int, types.Any)
 def int_impl(context, builder, sig, args):
     [ty] = sig.args
     [val] = args
@@ -1286,8 +1287,7 @@ def int_impl(context, builder, sig, args):
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
 
-@builtin
-@implement(float, types.Any)
+@lower_builtin(float, types.Any)
 def float_impl(context, builder, sig, args):
     [ty] = sig.args
     [val] = args
@@ -1295,8 +1295,7 @@ def float_impl(context, builder, sig, args):
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
 
-@builtin
-@implement(complex, types.VarArg(types.Any))
+@lower_builtin(complex, types.VarArg(types.Any))
 def complex_impl(context, builder, sig, args):
     complex_type = sig.return_type
     float_type = complex_type.underlying_float
@@ -1324,25 +1323,99 @@ def complex_impl(context, builder, sig, args):
     res = cmplx._getvalue()
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
-# -----------------------------------------------------------------------------
 
-@builtin_attr
-@impl_attribute(types.Module(math), "pi", types.float64)
-def math_pi_impl(context, builder, typ, value):
-    res = context.get_constant(types.float64, math.pi)
-    return impl_ret_untracked(context, builder, typ, res)
+@lower_builtin(types.NumberClass, types.Any)
+def number_constructor(context, builder, sig, args):
+    """
+    Convert any number to any other.
+    """
+    [val] = args
+    [valty] = sig.args
+    return context.cast(builder, val, valty, sig.return_type)
 
 
-@builtin_attr
-@impl_attribute(types.Module(math), "e", types.float64)
-def math_e_impl(context, builder, typ, value):
-    res = context.get_constant(types.float64, math.e)
-    return impl_ret_untracked(context, builder, typ, res)
+#-------------------------------------------------------------------------------
+# Implicit casts between numerics
+
+@lower_cast(types.Integer, types.Integer)
+def integer_to_integer(context, builder, fromty, toty, val):
+    if toty.bitwidth == fromty.bitwidth:
+        # Just a change of signedness
+        return val
+    elif toty.bitwidth < fromty.bitwidth:
+        # Downcast
+        return builder.trunc(val, context.get_value_type(toty))
+    elif fromty.signed:
+        # Signed upcast
+        return builder.sext(val, context.get_value_type(toty))
+    else:
+        # Unsigned upcast
+        return builder.zext(val, context.get_value_type(toty))
+
+@lower_cast(types.Integer, types.voidptr)
+def integer_to_voidptr(context, builder, fromty, toty, val):
+    return builder.inttoptr(val, context.get_value_type(toty))
+
+@lower_cast(types.Float, types.Float)
+def float_to_float(context, builder, fromty, toty, val):
+    lty = context.get_value_type(toty)
+    if fromty.bitwidth < toty.bitwidth:
+        return builder.fpext(val, lty)
+    else:
+        return builder.fptrunc(val, lty)
+
+@lower_cast(types.Integer, types.Float)
+def integer_to_float(context, builder, fromty, toty, val):
+    lty = context.get_value_type(toty)
+    if fromty.signed:
+        return builder.sitofp(val, lty)
+    else:
+        return builder.uitofp(val, lty)
+
+@lower_cast(types.Float, types.Integer)
+def float_to_integer(context, builder, fromty, toty, val):
+    lty = context.get_value_type(toty)
+    if toty.signed:
+        return builder.fptosi(val, lty)
+    else:
+        return builder.fptoui(val, lty)
+
+@lower_cast(types.Float, types.Complex)
+@lower_cast(types.Integer, types.Complex)
+def non_complex_to_complex(context, builder, fromty, toty, val):
+    real = context.cast(builder, val, fromty, toty.underlying_float)
+    imag = context.get_constant(toty.underlying_float, 0)
+
+    cmplx = context.make_complex(toty)(context, builder)
+    cmplx.real = real
+    cmplx.imag = imag
+    return cmplx._getvalue()
+
+@lower_cast(types.Complex, types.Complex)
+def complex_to_complex(context, builder, fromty, toty, val):
+    srccls, srcty = get_complex_info(fromty)
+    dstcls, dstty = get_complex_info(toty)
+
+    src = srccls(context, builder, value=val)
+    dst = dstcls(context, builder)
+    dst.real = context.cast(builder, src.real, srcty, dstty)
+    dst.imag = context.cast(builder, src.imag, srcty, dstty)
+    return dst._getvalue()
+
+@lower_cast(types.Any, types.Boolean)
+def any_to_boolean(context, builder, fromty, toty, val):
+    return context.is_true(builder, fromty, val)
+
+@lower_cast(types.Boolean, types.Any)
+def boolean_to_any(context, builder, fromty, toty, val):
+    # Casting from boolean to anything first casts to int32
+    asint = builder.zext(val, Type.int())
+    return context.cast(builder, asint, types.int32, toty)
+
 
 # -----------------------------------------------------------------------------
 
-@builtin
-@implement(intrinsics.array_ravel, types.Kind(types.Array))
+@lower_builtin(intrinsics.array_ravel, types.Array)
 def array_ravel_impl(context, builder, sig, args):
     [arrty] = sig.args
     [arr] = args
@@ -1372,8 +1445,7 @@ def array_ravel_impl(context, builder, sig, args):
 
 # -----------------------------------------------------------------------------
 
-@builtin
-@implement(type, types.Any)
+@lower_builtin(type, types.Any)
 def type_impl(context, builder, sig, args):
     """
     One-argument type() builtin.
