@@ -6,26 +6,26 @@ Usually due to invalid type conversion between function boundaries.
 from __future__ import print_function, division, absolute_import
 
 from numba import int32, int64
-from numba import njit
+from numba import jit
 from numba import unittest_support as unittest
 from .support import TestCase
 
 
-@njit
+@jit(nopython=True)
 def f_inner(a, b, c):
     return a, b, c
 
 def f(x, y, z):
     return f_inner(x, c=y, b=z)
 
-@njit
+@jit(nopython=True)
 def g_inner(a, b=2, c=3):
     return a, b, c
 
 def g(x, y, z):
     return g_inner(x, b=y), g_inner(a=z, c=x)
 
-@njit
+@jit(nopython=True)
 def star_inner(a=5, *b):
     return a, b
 
@@ -35,7 +35,7 @@ def star(x, y, z):
 def star_call(x, y, z):
     return star_inner(x, *y), star_inner(*z)
 
-@njit
+@jit(nopython=True)
 def argcast_inner(a, b):
     if b:
         # Here `a` is unified to int64 (from int32 originally)
@@ -48,20 +48,21 @@ def argcast(a, b):
 
 class TestNestedCall(TestCase):
 
-    def compile_func(self, pyfunc):
+    def compile_func(self, pyfunc, objmode=False):
         def check(*args, **kwargs):
             expected = pyfunc(*args, **kwargs)
             result = f(*args, **kwargs)
             self.assertPreciseEqual(result, expected)
-        f = njit(pyfunc)
+        flags = dict(forceobj=True) if objmode else dict(nopython=True)
+        f = jit(**flags)(pyfunc)
         return f, check
 
     def test_boolean_return(self):
-        @njit
+        @jit(nopython=True)
         def inner(x):
             return not x
 
-        @njit
+        @jit(nopython=True)
         def outer(x):
             if inner(x):
                 return True
@@ -71,22 +72,28 @@ class TestNestedCall(TestCase):
         self.assertFalse(outer(True))
         self.assertTrue(outer(False))
 
-    def test_named_args(self):
+    def test_named_args(self, objmode=False):
         """
         Test a nested function call with named (keyword) arguments.
         """
-        cfunc, check = self.compile_func(f)
+        cfunc, check = self.compile_func(f, objmode)
         check(1, 2, 3)
         check(1, y=2, z=3)
 
-    def test_default_args(self):
+    def test_named_args_objmode(self):
+        self.test_named_args(objmode=True)
+
+    def test_default_args(self, objmode=False):
         """
         Test a nested function call using default argument values.
         """
-        cfunc, check = self.compile_func(g)
+        cfunc, check = self.compile_func(g, objmode)
         check(1, 2, 3)
         check(1, y=2, z=3)
 
+    def test_default_args_objmode(self):
+        self.test_default_args(objmode=True)
+
     def test_star_args(self):
         """
         Test a nested function call to a function with *args in its signature.
@@ -94,13 +101,16 @@ class TestNestedCall(TestCase):
         cfunc, check = self.compile_func(star)
         check(1, 2, 3)
 
-    def test_star_call(self):
+    def test_star_call(self, objmode=False):
         """
         Test a function call with a *args.
         """
-        cfunc, check = self.compile_func(star_call)
+        cfunc, check = self.compile_func(star_call, objmode)
         check(1, (2,), (3,))
 
+    def test_star_call_objmode(self):
+        self.test_star_call(objmode=True)
+
     def test_argcast(self):
         """
         Issue #1488: implicitly casting an argument variable should not
