@@ -19,6 +19,16 @@ def get_b(ary, i):
 def get_c(ary, i):
     return ary[i].c
 
+def make_getitem(item):
+    # This also exercises constant lookup from a closure variable
+    def get_xx(ary, i):
+        return ary[i][item]
+    return get_xx
+
+getitem_a = make_getitem('a')
+getitem_b = make_getitem('b')
+getitem_c = make_getitem('c')
+
 def get_a_subarray(ary, i):
     return ary.a[i]
 
@@ -28,6 +38,15 @@ def get_b_subarray(ary, i):
 def get_c_subarray(ary, i):
     return ary.c[i]
 
+def make_getitem_subarray(item):
+    # This also exercises constant lookup from a closure variable
+    def get_xx_subarray(ary, i):
+        return ary[item][i]
+    return get_xx_subarray
+
+getitem_a_subarray = make_getitem_subarray('a')
+getitem_b_subarray = make_getitem_subarray('b')
+getitem_c_subarray = make_getitem_subarray('c')
 
 def get_two_arrays_a(ary1, ary2, i):
     return ary1[i].a + ary2[i].a
@@ -52,6 +71,15 @@ def set_b(ary, i, v):
 def set_c(ary, i, v):
     ary[i].c = v
 
+def make_setitem(item):
+    def set_xx(ary, i, v):
+        ary[i][item] = v
+    return set_xx
+
+setitem_a = make_setitem('a')
+setitem_b = make_setitem('b')
+setitem_c = make_setitem('c')
+
 def set_a_subarray(ary, i, v):
     ary.a[i] = v
 
@@ -61,6 +89,15 @@ def set_b_subarray(ary, i, v):
 def set_c_subarray(ary, i, v):
     ary.c[i] = v
 
+def make_setitem_subarray(item):
+    def set_xx_subarray(ary, i, v):
+        ary[item][i] = v
+    return set_xx_subarray
+
+setitem_a_subarray = make_setitem('a')
+setitem_b_subarray = make_setitem('b')
+setitem_c_subarray = make_setitem('c')
+
 def set_record(ary, i, j):
     ary[i] = ary[j]
 
@@ -269,14 +306,20 @@ class TestRecordDtype(unittest.TestCase):
     def test_get_a(self):
         self._test_get_equal(get_a)
         self._test_get_equal(get_a_subarray)
+        self._test_get_equal(getitem_a)
+        self._test_get_equal(getitem_a_subarray)
 
     def test_get_b(self):
         self._test_get_equal(get_b)
         self._test_get_equal(get_b_subarray)
+        self._test_get_equal(getitem_b)
+        self._test_get_equal(getitem_b_subarray)
 
     def test_get_c(self):
         self._test_get_equal(get_c)
         self._test_get_equal(get_c_subarray)
+        self._test_get_equal(getitem_c)
+        self._test_get_equal(getitem_c_subarray)
 
     def _test_get_two_equal(self, pyfunc):
         '''
@@ -331,6 +374,8 @@ class TestRecordDtype(unittest.TestCase):
             self._test_set_equal(pyfunc, 3., types.float32)
         check(set_a)
         check(set_a_subarray)
+        check(setitem_a)
+        check(setitem_a_subarray)
 
     def test_set_b(self):
         def check(pyfunc):
@@ -339,6 +384,8 @@ class TestRecordDtype(unittest.TestCase):
             self._test_set_equal(pyfunc, 123, types.float64)
         check(set_b)
         check(set_b_subarray)
+        check(setitem_b)
+        check(setitem_b_subarray)
 
     def test_set_c(self):
         def check(pyfunc):
@@ -347,6 +394,8 @@ class TestRecordDtype(unittest.TestCase):
             self._test_set_equal(pyfunc, 43j, types.complex128)
         check(set_c)
         check(set_c_subarray)
+        check(setitem_c)
+        check(setitem_c_subarray)
 
     def test_set_record(self):
         pyfunc = set_record
