@@ -61,7 +61,6 @@ class Slice(ConcreteTemplate):
     ]
 
 
-@infer_global(range)
 class Range(ConcreteTemplate):
     cases = [
         signature(types.range_state32_type, types.int32),
@@ -78,6 +77,9 @@ class Range(ConcreteTemplate):
                   types.uint64),
     ]
 
+for func in RANGE_ITER_OBJECTS:
+    infer_global(func, typing_key=range)(Range)
+
 
 @infer
 class GetIter(AbstractTemplate):
