@@ -7,26 +7,29 @@ from .templates import (AttributeTemplate, AbstractTemplate, CallableTemplate,
 
 from ..numpy_support import (ufunc_find_matching_loop,
                              supported_ufunc_loop, as_dtype,
-                             from_dtype)
+                             from_dtype, resolve_output_type)
 from ..numpy_support import version as numpy_version
 
 from ..errors import TypingError
 
 registry = Registry()
-builtin = registry.register
-builtin_global = registry.register_global
-builtin_attr = registry.register_attr
-
-
-@builtin_attr
-class NumpyModuleAttribute(AttributeTemplate):
-    # note: many unary ufuncs are added later on, using setattr
-    key = types.Module(numpy)
+infer = registry.register
+infer_global = registry.register_global
+infer_getattr = registry.register_attr
 
 
 class Numpy_rules_ufunc(AbstractTemplate):
     @classmethod
     def _handle_inputs(cls, ufunc, args, kws):
+        """
+        Process argument types to a given *ufunc*.
+        Returns a (base types, explicit outputs, ndims, layout) tuple where:
+        - `base types` is a tuple of scalar types for each input
+        - `explicit outputs` is a tuple of explicit output types (arrays)
+        - `ndims` is the number of dimensions of the loop and also of
+          any outputs, explicit or implicit
+        - `layout` is the layout for any implicit output to be allocated
+        """
         nin = ufunc.nin
         nout = ufunc.nout
         nargs = ufunc.nargs
@@ -46,31 +49,34 @@ class Numpy_rules_ufunc(AbstractTemplate):
             msg = "ufunc '{0}': too many arguments ({1} found, {2} maximum)"
             raise TypingError(msg=msg.format(ufunc.__name__, len(args), nargs))
 
-
-        arg_ndims = [a.ndim if isinstance(a, types.Array) else 0 for a in args]
+        args = [a.as_array if isinstance(a, types.ArrayCompatible) else a
+                for a in args]
+        arg_ndims = [a.ndim if isinstance(a, types.ArrayCompatible) else 0
+                     for a in args]
         ndims = max(arg_ndims)
 
         # explicit outputs must be arrays (no explicit scalar return values supported)
         explicit_outputs = args[nin:]
 
         # all the explicit outputs must match the number max number of dimensions
-        if not all((d == ndims for d in arg_ndims[nin:])):
+        if not all(d == ndims for d in arg_ndims[nin:]):
             msg = "ufunc '{0}' called with unsuitable explicit output arrays."
             raise TypingError(msg=msg.format(ufunc.__name__))
 
-        if not all((isinstance(output, types.Array) for output in explicit_outputs)):
+        if not all(isinstance(output, types.ArrayCompatible)
+                   for output in explicit_outputs):
             msg = "ufunc '{0}' called with an explicit output that is not an array"
             raise TypingError(msg=msg.format(ufunc.__name__))
 
         # find the kernel to use, based only in the input types (as does NumPy)
-        base_types = [x.dtype if isinstance(x, types.Array) else x
+        base_types = [x.dtype if isinstance(x, types.ArrayCompatible) else x
                       for x in args]
 
         # Figure out the output array layout, if needed.
         layout = None
         if ndims > 0 and (len(explicit_outputs) < ufunc.nout):
             layout = 'C'
-            layouts = [x.layout if isinstance(x, types.Array) else ''
+            layouts = [x.layout if isinstance(x, types.ArrayCompatible) else ''
                        for x in args]
 
             # Prefer C contig if any array is C contig.
@@ -120,6 +126,8 @@ class Numpy_rules_ufunc(AbstractTemplate):
                 assert layout is not None
                 ret_tys = [types.Array(dtype=ret_ty, ndim=ndims, layout=layout)
                            for ret_ty in ret_tys]
+                ret_tys = [resolve_output_type(self.context, args, ret_ty)
+                           for ret_ty in ret_tys]
             out.extend(ret_tys)
 
         # note: although the previous code should support multiple return values, only one
@@ -129,7 +137,7 @@ class Numpy_rules_ufunc(AbstractTemplate):
         return signature(*out)
 
 
-@builtin
+@infer
 class UnaryPositiveArray(AbstractTemplate):
     '''Typing template class for +(array) expressions.  This operator is
     special because there is no Numpy ufunc associated with it; we
@@ -140,7 +148,7 @@ class UnaryPositiveArray(AbstractTemplate):
 
     def generic(self, args, kws):
         assert not kws
-        if len(args) == 1 and isinstance(args[0], types.Array):
+        if len(args) == 1 and isinstance(args[0], types.ArrayCompatible):
             arg_ty = args[0]
             return arg_ty.copy()(arg_ty)
 
@@ -175,7 +183,7 @@ class NumpyRulesArrayOperator(Numpy_rules_ufunc):
     @classmethod
     def install_operations(cls):
         for op, ufunc_name in cls._op_map.items():
-            builtin(type("NumpyRulesArrayOperator_" + ufunc_name, (cls,),
+            infer(type("NumpyRulesArrayOperator_" + ufunc_name, (cls,),
                          dict(key=op)))
 
     def generic(self, args, kws):
@@ -216,7 +224,7 @@ class NumpyRulesInplaceArrayOperator(NumpyRulesArrayOperator):
         # (for example int8[:] += int16[:] should use an int8[:] output,
         #  not int16[:])
         lhs, rhs = args
-        if not isinstance(lhs, types.Array):
+        if not isinstance(lhs, types.ArrayCompatible):
             return
         args = args + (lhs,)
         sig = super(NumpyRulesInplaceArrayOperator, self).generic(args, kws)
@@ -237,7 +245,7 @@ class NumpyRulesUnaryArrayOperator(NumpyRulesArrayOperator):
 
     def generic(self, args, kws):
         assert not kws
-        if len(args) == 1 and isinstance(args[0], types.Array):
+        if len(args) == 1 and isinstance(args[0], types.ArrayCompatible):
             return super(NumpyRulesUnaryArrayOperator, self).generic(args, kws)
 
 
@@ -285,11 +293,11 @@ _unsupported = set([ 'frexp', # this one is tricky, as it has 2 returns
                      'modf',  # this one also has 2 returns
                  ])
 
-# a list of ufuncs that are in fact aliases of other ufuncs. They need to insert the
+# A list of ufuncs that are in fact aliases of other ufuncs. They need to insert the
 # resolve method, but not register the ufunc itself
 _aliases = set(["bitwise_not", "mod", "abs"])
 
-#in python3 numpy.divide is mapped to numpy.true_divide
+# In python3 numpy.divide is mapped to numpy.true_divide
 if numpy.divide == numpy.true_divide:
     _aliases.add("divide")
 
@@ -299,12 +307,9 @@ def _numpy_ufunc(name):
         key = func
 
     typing_class.__name__ = "resolve_{0}".format(name)
-    # Add the resolve method to NumpyModuleAttribute
-    setattr(NumpyModuleAttribute, typing_class.__name__,
-            lambda s, m: types.Function(typing_class))
 
     if not name in _aliases:
-        builtin_global(func, types.Function(typing_class))
+        infer_global(func, types.Function(typing_class))
 
 all_ufuncs = sum([_math_operations, _trigonometric_functions,
                   _bit_twiddling_functions, _comparison_functions,
@@ -348,7 +353,7 @@ class Numpy_method_redirection(AbstractTemplate):
         # Resolve arguments on the bound function
         meth_sig = self.context.resolve_function_type(meth_ty, args[1:], kws)
         if meth_sig is not None:
-            return signature(meth_sig.return_type, meth_sig.recvr, *meth_sig.args)
+            return meth_sig.as_function()
 
 
 # Function to glue attributes onto the numpy-esque object
@@ -356,7 +361,7 @@ def _numpy_redirect(fname):
     numpy_function = getattr(numpy, fname)
     cls = type("Numpy_redirect_{0}".format(fname), (Numpy_method_redirection,),
                dict(key=numpy_function, method_name=fname))
-    builtin_global(numpy_function, types.Function(cls))
+    infer_global(numpy_function, types.Function(cls))
 
 for func in ['min', 'max', 'sum', 'prod', 'mean', 'median', 'var', 'std',
              'cumsum', 'cumprod', 'argmin', 'argmax', 'nonzero']:
@@ -383,7 +388,7 @@ def register_number_classes(register_global):
         register_global(np_type, types.NumberClass(nb_type))
 
 
-register_number_classes(builtin_global)
+register_number_classes(infer_global)
 
 
 # -----------------------------------------------------------------------------
@@ -403,6 +408,9 @@ def _parse_dtype(dtype):
         return dtype.dtype
 
 
+@infer_global(numpy.empty)
+@infer_global(numpy.zeros)
+@infer_global(numpy.ones)
 class NdConstructor(CallableTemplate):
     """
     Typing template for np.empty(), .zeros(), .ones().
@@ -422,67 +430,43 @@ class NdConstructor(CallableTemplate):
         return typer
 
 
+@infer_global(numpy.empty_like)
+@infer_global(numpy.zeros_like)
 class NdConstructorLike(CallableTemplate):
     """
     Typing template for np.empty_like(), .zeros_like(), .ones_like().
     """
 
     def generic(self):
-        def typer(arr, dtype=None):
-            if dtype is None:
-                nb_dtype = arr.dtype
-            else:
+        """
+        np.empty_like(array) -> empty array of the same shape and layout
+        np.empty_like(scalar) -> empty 0-d array of the scalar type
+        """
+        def typer(arg, dtype=None):
+            if dtype is not None:
                 nb_dtype = _parse_dtype(dtype)
+            elif isinstance(arg, types.Array):
+                nb_dtype = arg.dtype
+            else:
+                nb_dtype = arg
             if nb_dtype is not None:
-                layout = arr.layout if arr.layout != 'A' else 'C'
-                return arr.copy(dtype=nb_dtype, layout=layout)
+                if isinstance(arg, types.Array):
+                    layout = arg.layout if arg.layout != 'A' else 'C'
+                    return arg.copy(dtype=nb_dtype, layout=layout)
+                else:
+                    return types.Array(nb_dtype, 0, 'C')
 
         return typer
 
 
-@builtin
-class NdEmpty(NdConstructor):
-    key = numpy.empty
-
-@builtin
-class NdZeros(NdConstructor):
-    key = numpy.zeros
-
-@builtin
-class NdOnes(NdConstructor):
-    key = numpy.ones
-    return_new_reference = True
-
-@builtin
-class NdEmptyLike(NdConstructorLike):
-    key = numpy.empty_like
-
-@builtin
-class NdZerosLike(NdConstructorLike):
-    key = numpy.zeros_like
-
-
-builtin_global(numpy.empty, types.Function(NdEmpty))
-builtin_global(numpy.zeros, types.Function(NdZeros))
-builtin_global(numpy.ones, types.Function(NdOnes))
-builtin_global(numpy.empty_like, types.Function(NdEmptyLike))
-builtin_global(numpy.zeros_like, types.Function(NdZerosLike))
-
 if numpy_version >= (1, 7):
     # In Numpy 1.6, ones_like() was a ufunc and had a different signature.
-    @builtin
-    class NdOnesLike(NdConstructorLike):
-        key = numpy.ones_like
-
-    builtin_global(numpy.ones_like, types.Function(NdOnesLike))
+    infer_global(numpy.ones_like)(NdConstructorLike)
 
 
 if numpy_version >= (1, 8):
-    @builtin
+    @infer_global(numpy.full)
     class NdFull(CallableTemplate):
-        key = numpy.full
-        return_new_reference = True
-
 
         def generic(self):
             def typer(shape, fill_value, dtype=None):
@@ -497,30 +481,32 @@ if numpy_version >= (1, 8):
 
             return typer
 
-    @builtin
+    @infer_global(numpy.full_like)
     class NdFullLike(CallableTemplate):
-        key = numpy.full_like
-        return_new_reference = True
 
         def generic(self):
-            def typer(arr, fill_value, dtype=None):
-                if dtype is None:
-                    nb_dtype = arr.dtype
-                else:
+            """
+            np.full_like(array, val) -> array of the same shape and layout
+            np.full_like(scalar, val) -> 0-d array of the scalar type
+            """
+            def typer(arg, fill_value, dtype=None):
+                if dtype is not None:
                     nb_dtype = _parse_dtype(dtype)
+                elif isinstance(arg, types.Array):
+                    nb_dtype = arg.dtype
+                else:
+                    nb_dtype = arg
                 if nb_dtype is not None:
-                    return arr.copy(dtype=nb_dtype)
+                    if isinstance(arg, types.Array):
+                        return arg.copy(dtype=nb_dtype)
+                    else:
+                        return types.Array(dtype=nb_dtype, ndim=0, layout='C')
 
             return typer
 
-    builtin_global(numpy.full, types.Function(NdFull))
-    builtin_global(numpy.full_like, types.Function(NdFullLike))
 
-
-@builtin
+@infer_global(numpy.identity)
 class NdIdentity(AbstractTemplate):
-    key = numpy.identity
-    return_new_reference = True
 
     def generic(self, args, kws):
         assert not kws
@@ -536,17 +522,13 @@ class NdIdentity(AbstractTemplate):
             return_type = types.Array(ndim=2, dtype=nb_dtype, layout='C')
             return signature(return_type, *args)
 
-builtin_global(numpy.identity, types.Function(NdIdentity))
-
 
 def _infer_dtype_from_inputs(inputs):
     return dtype
 
 
-@builtin
+@infer_global(numpy.eye)
 class NdEye(CallableTemplate):
-    key = numpy.eye
-    return_new_reference = True
 
     def generic(self):
         def typer(N, M=None, k=None, dtype=None):
@@ -559,13 +541,9 @@ class NdEye(CallableTemplate):
 
         return typer
 
-builtin_global(numpy.eye, types.Function(NdEye))
 
-
-@builtin
+@infer_global(numpy.arange)
 class NdArange(AbstractTemplate):
-    key = numpy.arange
-    return_new_reference = True
 
     def generic(self, args, kws):
         assert not kws
@@ -585,13 +563,9 @@ class NdArange(AbstractTemplate):
         return_type = types.Array(ndim=1, dtype=dtype, layout='C')
         return signature(return_type, *args)
 
-builtin_global(numpy.arange, types.Function(NdArange))
 
-
-@builtin
+@infer_global(numpy.linspace)
 class NdLinspace(AbstractTemplate):
-    key = numpy.linspace
-    return_new_reference = True
 
     def generic(self, args, kws):
         assert not kws
@@ -613,12 +587,9 @@ class NdLinspace(AbstractTemplate):
         return_type = types.Array(ndim=1, dtype=dtype, layout='C')
         return signature(return_type, *args)
 
-builtin_global(numpy.linspace, types.Function(NdLinspace))
 
-
-@builtin
+@infer_global(numpy.frombuffer)
 class NdFromBuffer(CallableTemplate):
-    key = numpy.frombuffer
 
     def generic(self):
         def typer(buffer, dtype=None):
@@ -635,12 +606,9 @@ class NdFromBuffer(CallableTemplate):
 
         return typer
 
-builtin_global(numpy.frombuffer, types.Function(NdFromBuffer))
 
-
-@builtin
+@infer_global(numpy.sort)
 class NdSort(CallableTemplate):
-    key = numpy.sort
 
     def generic(self):
         def typer(a):
@@ -649,15 +617,120 @@ class NdSort(CallableTemplate):
 
         return typer
 
-builtin_global(numpy.sort, types.Function(NdSort))
+
+# -----------------------------------------------------------------------------
+# Linear algebra
+
+
+class MatMulTyperMixin(object):
+
+    def matmul_typer(self, a, b, out=None):
+        """
+        Typer function for Numpy matrix multiplication.
+        """
+        if not isinstance(a, types.Array) or not isinstance(b, types.Array):
+            return
+        if not all(x.ndim in (1, 2) for x in (a, b)):
+            raise TypingError("%s only supported on 1-D and 2-D arrays"
+                              % (self.func_name, ))
+        # Output dimensionality
+        ndims = set([a.ndim, b.ndim])
+        if ndims == set([2]):
+            # M * M
+            out_ndim = 2
+        elif ndims == set([1, 2]):
+            # M* V and V * M
+            out_ndim = 1
+        elif ndims == set([1]):
+            # V * V
+            out_ndim = 0
+
+        if out is not None:
+            if out_ndim == 0:
+                raise TypeError("explicit output unsupported for vector * vector")
+            elif out.ndim != out_ndim:
+                raise TypeError("explicit output has incorrect dimensionality")
+            if not isinstance(out, types.Array) or out.layout != 'C':
+                raise TypeError("output must be a C-contiguous array")
+            all_args = (a, b, out)
+        else:
+            all_args = (a, b)
+
+        if not all(x.layout in 'CF' for x in (a, b)):
+            # Numpy seems to allow non-contiguous arguments, but we
+            # don't (we would need to copy before calling BLAS)
+            raise TypingError("%s only supported on "
+                              "contiguous arrays" % (self.func_name,))
+        if not all(x.dtype == a.dtype for x in all_args):
+            raise TypingError("%s arguments must all have "
+                              "the same dtype" % (self.func_name,))
+        if not isinstance(a.dtype, (types.Float, types.Complex)):
+            raise TypingError("%s only supported on "
+                              "float and complex arrays"
+                              % (self.func_name,))
+        if out:
+            return out
+        elif out_ndim > 0:
+            return types.Array(a.dtype, out_ndim, 'C')
+        else:
+            return a.dtype
+
+
+@infer_global(numpy.dot)
+class Dot(MatMulTyperMixin, CallableTemplate):
+    func_name = "np.dot()"
+
+    def generic(self):
+        def typer(a, b, out=None):
+            # NOTE: np.dot() and the '@' operator have distinct semantics
+            # for >2-D arrays, but we don't support them.
+            return self.matmul_typer(a, b, out)
+
+        return typer
+
+
+@infer_global(numpy.vdot)
+class VDot(CallableTemplate):
+
+    def generic(self):
+        def typer(a, b):
+            if not isinstance(a, types.Array) or not isinstance(b, types.Array):
+                return
+            if not all(x.ndim == 1 for x in (a, b)):
+                raise TypingError("np.vdot() only supported on 1-D arrays")
+            if not all(x.layout in 'CF' for x in (a, b)):
+                # Numpy seems to allow non-contiguous arguments, but we
+                # don't (we would need to copy before calling BLAS)
+                raise TypingError("np.vdot() only supported on "
+                                  "contiguous arrays")
+            if not all(x.dtype == a.dtype for x in (a, b)):
+                raise TypingError("np.vdot() arguments must all have "
+                                  "the same dtype")
+            if not isinstance(a.dtype, (types.Float, types.Complex)):
+                raise TypingError("np.vdot() only supported on "
+                                  "float and complex arrays")
+            return a.dtype
+
+        return typer
+
+
+@infer
+class MatMul(MatMulTyperMixin, AbstractTemplate):
+    key = "@"
+    func_name = "'@'"
+
+    def generic(self, args, kws):
+        assert not kws
+        restype = self.matmul_typer(*args)
+        if restype is not None:
+            return signature(restype, *args)
 
 
 # -----------------------------------------------------------------------------
 # Miscellaneous functions
 
-@builtin
+@infer_global(numpy.ndenumerate)
 class NdEnumerate(AbstractTemplate):
-    key = numpy.ndenumerate
 
     def generic(self, args, kws):
         assert not kws
@@ -667,11 +740,9 @@ class NdEnumerate(AbstractTemplate):
             enumerate_type = types.NumpyNdEnumerateType(arr)
             return signature(enumerate_type, *args)
 
-builtin_global(numpy.ndenumerate, types.Function(NdEnumerate))
 
-@builtin
+@infer_global(numpy.ndindex)
 class NdIndex(AbstractTemplate):
-    key = numpy.ndindex
 
     def generic(self, args, kws):
         assert not kws
@@ -690,12 +761,12 @@ class NdIndex(AbstractTemplate):
             iterator_type = types.NumpyNdIndexType(len(shape))
             return signature(iterator_type, *args)
 
-builtin_global(numpy.ndindex, types.Function(NdIndex))
 
-
-@builtin
+# We use the same typing key for np.round() and np.around() to
+# re-use the implementations automatically.
+@infer_global(numpy.round)
+@infer_global(numpy.around, typing_key=numpy.round)
 class Round(AbstractTemplate):
-    key = numpy.round
 
     def generic(self, args, kws):
         assert not kws
@@ -724,32 +795,59 @@ class Round(AbstractTemplate):
                 or isinstance(out.dtype, types.Complex)):
                 return signature(out, *args)
 
-builtin_global(numpy.round, types.Function(Round))
-builtin_global(numpy.around, types.Function(Round))
 
-
-@builtin
+@infer_global(numpy.where)
 class Where(AbstractTemplate):
-    key = numpy.where
 
     def generic(self, args, kws):
         assert not kws
 
         if len(args) == 1:
-            # np.where(cond) is the same as np.nonzero(cond)
+            # 0-dim arrays return one result array
             ary = args[0]
             ndim = max(ary.ndim, 1)
             retty = types.UniTuple(types.Array(types.intp, 1, 'C'), ndim)
             return signature(retty, ary)
 
         elif len(args) == 3:
+            # NOTE: contrary to Numpy, we only support homogenous arguments
             cond, x, y = args
-            if (cond.ndim == x.ndim == y.ndim and
-                x.dtype == y.dtype):
-                retty = types.Array(x.dtype, x.ndim, x.layout)
-                return signature(retty, *args)
-
-builtin_global(numpy.where, types.Function(Where))
+            if isinstance(cond, types.Array):
+                # array where()
+                if (cond.ndim == x.ndim == y.ndim and
+                    x.dtype == y.dtype):
+                    retty = types.Array(x.dtype, x.ndim, x.layout)
+                    return signature(retty, *args)
+            else:
+                # scalar where()
+                if not isinstance(x, types.Array) and x == y:
+                    retty = types.Array(x, 0, 'C')
+                    return signature(retty, *args)
 
 
-builtin_global(numpy, types.Module(numpy))
+@infer_global(numpy.sinc)
+class Sinc(AbstractTemplate):
+
+    def generic(self, args, kws):
+        assert not kws
+        assert len(args) == 1
+        arg = args[0]
+        supported_scalars = (types.Float, types.Complex)
+        if (isinstance(arg, supported_scalars) or
+              (isinstance(arg, types.Array) and
+               isinstance(arg.dtype, supported_scalars))):
+            return signature(arg, arg)
+
+
+@infer_global(numpy.angle)
+class Angle(CallableTemplate):
+    """
+    Typing template for np.angle()
+    """
+    def generic(self):
+        def typer(ref, deg = False):
+            if isinstance(ref, types.Array):
+                return ref.copy(dtype=ref.underlying_float)
+            else:
+                return types.float64
+        return typer
