@@ -10,8 +10,8 @@ from numba.typing.templates import (ConcreteTemplate, AbstractTemplate,
                                     signature, Registry)
 
 registry = Registry()
-builtin_attr = registry.register_attr
-builtin_global = registry.register_global
+infer_getattr = registry.register_attr
+infer_global = registry.register_global
 
 
 class MappedOperator(AbstractTemplate):
@@ -39,7 +39,7 @@ for name, inplace_name, op in utils.operator_map:
     op_func = getattr(operator, name)
     op_type = type('Operator_' + name, (MappedOperator,),
                    {'key': op_func, 'op': op})
-    builtin_global(op_func, types.Function(op_type))
+    infer_global(op_func, types.Function(op_type))
 
     if inplace_name:
         op_func = getattr(operator, inplace_name)
@@ -47,7 +47,4 @@ for name, inplace_name, op in utils.operator_map:
                        {'key': op_func,
                         'mutable_op': op + '=',
                         'immutable_op': op})
-        builtin_global(op_func, types.Function(op_type))
-
-
-builtin_global(operator, types.Module(operator))
+        infer_global(op_func, types.Function(op_type))
